@@ -201,7 +201,7 @@ class PythonExpression:
     def __str__(self):
         return self.definition
 
-    def __call__(self, mapping=None, **values):
+    def __call__(self, mapping: dict = None, **values):
         if mapping is not None:
             values, tmp = mapping, values
             values.update(tmp)
@@ -235,10 +235,10 @@ class SliceAgain(LazyEvaluation):
         expression = self.expr
         if pending(expression):
             expression = expression(data).decode('utf8')
-        return sliceobj(expression, metavars(data))
+        return sliceobj(expression, data)
 
 
-def sliceobj(expression: Union[int, str, slice], variables: Optional[dict] = None, range=False) -> Union[slice, SliceAgain]:
+def sliceobj(expression: Union[int, str, slice], data: Optional[Chunk] = None, range=False, final=False) -> Union[slice, SliceAgain]:
     """
     Uses `refinery.lib.argformats.PythonExpression` to parse slice expressions
     where the bounds can be given as arithmetic expressions. For example, this
@@ -252,24 +252,30 @@ def sliceobj(expression: Union[int, str, slice], variables: Optional[dict] = Non
     if isinstance(expression, (bytes, bytearray)):
         expression = expression.decode('utf8')
 
-    if is_valid_variable_name(expression):
-        try:
-            return sliceobj(variables[expression], variables, range)
-        except (KeyError, TypeError):
-            return SliceAgain(expression)
+    if data is None:
+        variables = {}
+    else:
+        variables = metavars(data)
+        if is_valid_variable_name(expression):
+            try:
+                return sliceobj(variables[expression], data, final=True)
+            except Exception:
+                pass
 
-    expression = expression or ':'
-    sliced = expression.split(':')
+    sliced = expression and expression.split(':') or ['', '']
 
     if not sliced or len(sliced) > 3:
-        raise ArgumentTypeError(F'the expression {expression} is not a valid slice.')
+        raise ArgumentTypeError(F'the expression "{expression}" is not a valid slice.')
     try:
-        kwargs = variables or {}
-        sliced = [None if not t else PythonExpression.evaluate(t, kwargs) for t in sliced]
+        sliced = [None if not t else PythonExpression.evaluate(t, variables) for t in sliced]
     except ParserVariableMissing:
-        if variables is not None:
+        if final:
             raise
-        return SliceAgain(expression)
+        elif data is not None:
+            parser = DelayedNumSeqArgument(expression)
+            return sliceobj(parser(data), data, range, final=True)
+        else:
+            return SliceAgain(expression)
     if len(sliced) == 1:
         k = sliced[0]
         if not range:
@@ -628,6 +634,15 @@ class DelayedArgument(LazyEvaluation):
         import base64
         return base64.b16decode(string, casefold=True)
 
+    @handler.register('n')
+    def n(self, string: bytes) -> bytes:
+        """
+        The final modifier `n:string` returns the un-escaped version of a string containing
+        backslash escape sequences.
+        """
+        from refinery.units.encoding.esc import esc
+        return esc().process(string)
+
     @handler.register('q', 'Q', final=True)
     def q(self, string: str) -> bytes:
         """
@@ -669,7 +684,7 @@ class DelayedArgument(LazyEvaluation):
             if not region:
                 bounds = slice(0, None)
             else:
-                bounds = sliceobj(region, metavars(data), range=True)
+                bounds = sliceobj(region, data, range=True)
             if bounds.step:
                 raise ValueError('Step size is not supported for file slices.')
             with path.open('rb') as stream:
@@ -695,8 +710,12 @@ class DelayedArgument(LazyEvaluation):
         of bytes starting at zero.
         """
         def compute_range(data: Optional[Chunk] = None):
-            meta = data and getattr(data, 'meta', None) or {}
-            bounds = sliceobj(region, meta, range=True)
+            try:
+                bounds = sliceobj(region, data, range=True)
+            except ParserVariableMissing:
+                raise TooLazy
+            if pending(bounds):
+                raise TooLazy
             start = bounds.start or 0
             stop = bounds.stop
             step = bounds.step or 1
@@ -708,7 +727,7 @@ class DelayedArgument(LazyEvaluation):
             return result
         try:
             return compute_range()
-        except ParserVariableMissing:
+        except TooLazy:
             return compute_range
 
     @handler.register('c', 'copy', final=True)
@@ -718,7 +737,7 @@ class DelayedArgument(LazyEvaluation):
         as a `refinery.lib.argformats.sliceobj`. The result contains the corresponding slice
         of the input data.
         """
-        return lambda d: d[sliceobj(region, d.meta)]
+        return lambda d: d[sliceobj(region, d)]
 
     @handler.register('x', 'cut', final=True)
     def cut(self, region: str) -> bytes:
@@ -727,7 +746,7 @@ class DelayedArgument(LazyEvaluation):
         but the corresponding bytes are also removed from the input data.
         """
         def extract(data: Union[bytearray, Chunk]):
-            bounds = sliceobj(region, metavars(data))
+            bounds = sliceobj(region, data)
             result = bytearray(data[bounds])
             data[bounds] = []
             return result
@@ -970,7 +989,7 @@ class DelayedArgument(LazyEvaluation):
             @lru_cache(maxsize=512, typed=False)
             def accumulate(A):
                 return update(meta, A=A)
-            meta = metavars(data)
+            meta = dict(metavars(data))
             A = seed and seed(meta) or 0
             F = feed and feed(meta, A=A) or A
             S = skip
@@ -1025,7 +1044,7 @@ class DelayedArgument(LazyEvaluation):
         reduction = PythonExpression(reduction, all_variables_allowed=True)
 
         def finalize(data: Optional[Chunk] = None):
-            args = metavars(data)
+            args = dict(metavars(data))
             return reduce(lambda S, B: reduction(args, S=S, B=B), it, seed and seed(args) or 0)
 
         try:
