@@ -11,11 +11,16 @@ class xttar(ArchiveUnit):
     """
     Extract files from a Tar archive.
     """
-    def __init__(self, *paths, list=False, join_path=False, drop_path=False, path=b'path', date=b'date'):
-        super().__init__(*paths, list=list, join_path=join_path, drop_path=drop_path, path=path, date=date)
-
     def unpack(self, data: bytearray):
-        archive = tarfile.open(fileobj=MemoryFile(data))
+        with MemoryFile(data) as stream:
+            try:
+                archive = tarfile.open(fileobj=stream)
+            except Exception:
+                ustar = data.find(B'ustar')
+                if ustar < 257:
+                    raise
+                stream.seek(ustar - 257)
+                archive = tarfile.open(fileobj=stream)
         for info in archive.getmembers():
             if not info.isfile():
                 continue
@@ -29,5 +34,5 @@ class xttar(ArchiveUnit):
     def handles(cls, data: bytearray) -> bool:
         ustar = data.find(B'ustar')
         if ustar >= 0:
-            return data[ustar:ustar + 3] in (B'\x00\x30\x30', B'\x20\x20\x00')
+            return ustar == 257 or data[ustar:ustar + 3] in (B'\x00\x30\x30', B'\x20\x20\x00')
         return False
